# Copyright 2015 Eye Physics and Jay Watson
#
# Create Bolts, Screws, Washers, Tapped holes, drilled holes and nuts 
# in standard sizes as components, that should print as workable parts.
#
# 2.01.2015  Jay Watson, Eye Physics
# 4.07.2015  Jay Watson, V1.0.1 Fixed reversed face on phillips screws, and corrected HTML path per feedback from SketchUcation
#                               Forum.
# 4.13.2015  Jay Watson, V1.0.2 Fixed reverse face on Nut, and correctly handle decimal character "," and Added Arc Quality per request
#                               Also create small holes scaled by 10 and then shrunk because of Sketchups habit of dropping small faces.
#-----------------------------------------------------------------------------

require "sketchup.rb"
require_relative "EPFastenerConstants"

module EP
module EPFasteners

#=============================================================================
class EPTappedHole < EPFastenerConstants
#=============================================================================
#---------------------------------------------------------------------------------------------------------
#---------------------------------------------------------------------------------------------------------
   def self.create(modelunits)
      @@UNITS = modelunits
      if !defined? @@PREVUNITS
          @@PREVUNITS = "UNK"
      end

      @@ZERO = "0.0".to_f

      if defined? @@Boltsize
         if @@UNITS != @@PREVUNITS 
            remove_class_variable(:@@BOLTSIZE)		#If changing units reinitialize
            @@PREVUNITS = @@UNITS
         end
      end

      if !defined? @@BoltSize
         if @@UNITS == "SAE"
             @@BoltSize = "#10"
             @@Length = "1.00"
         else
             @@BoltSize = "M8"
             @@Length = "30"
         end
         @@ThruHole = "Yes"
         @@TPI = "Coarse"
         @@Head   = "Hex"
         @@Fullthread = "Yes"
         @@Inclwasher = "No"
	 @@Scale = "1.0".to_f
      end
      @@PREVUNITS = @@UNITS

      self.showDialog()
   end

#---------------------------------------------------------------------------------------------------------
   def self.continue
      model = Sketchup.active_model

         #Does this Hole exist in the model already?
         #
         #partname = "Tapped Hole #{@boltsize}..."
         defn = Sketchup.active_model.definitions[self.partname]
         if defn.nil?
            model.start_operation(self.partname, true)
            utshole = EPTappedHole.new
            container= utshole.create_entity(model)
            if container.nil?
               model.abort_operation
               return
            end
            utshole.create_entities(container)
            model.commit_operation
            defn = utshole.entity
         end
         #
         self.closeDialog   
         Sketchup.active_model.place_component(defn, false)

   end
#---------------------------------------------------------------------------------------------------------
#---------------------------------------------------------------------------------------------------------
   def self.InitializeDialog
      @@dialog = UI::WebDialog.new("SAE Tapped Hole", true, "", 550, 400, 200, 200, false )
      @@dialog.add_action_callback("ruby_OKPressed") {|dialog,params| self.performOK(params)}
      @@dialog.add_action_callback("ruby_CANCELPressed") {self.performCancel()}
      @@dialog.add_action_callback("ruby_Continue") {self.continue()}
      @@dialog.add_action_callback("ruby_DocumentReady") {self.performLoadVars()}
   end

#---------------------------------------------------------------------------------------------------------
def self.performLoadVars()

   js_command = "$(\"#BOLTSIZE\").val('#{@@BoltSize}')"
   @@dialog.execute_script(js_command)

   js_command = "setThreadType('#{@@TPI}')"
   @@dialog.execute_script(js_command)

   if @@UNITS == "SAE"
      js_command = "setLength('#{@@Length}')"
   else
      js_command = "setLength('#{@@Length.to_mm.round(6)}')"
   end
   @@dialog.execute_script(js_command)

   thruholetype = (@@ThruHole=="Yes" ? "ThruHole":"Hole")
   js_command = "setHoleType('#{thruholetype}')"

   @@dialog.execute_script(js_command)

end

#---------------------------------------------------------------------------------------------------------
def self.performOK(*params)
   p = params[0].split(";")
   p = p.map{|e| e.split("=")}
   p.each { |e|

      case e[0]
         when "bs"
            @@BoltSize = e[1]
         when "ln"
            @@Length = EPTappedHole::cLength(e[1], @@UNITS)
         when "ht"
            @@ThruHole = (e[1]=="ThruHole" ? "Yes" : "No")
         when "tt"
            @@TPI = e[1]
     end
   }

   die  = @@UTS.select {|s| s[0]==@@BoltSize && s[1]==@@TPI}

   @@Pitch = EPTappedHole::cLength(die[0][2], @@UNITS)
   @@Diam  = EPTappedHole::cLength(die[0][3], @@UNITS)
   @@Scale = (die[0][4]).to_f
   if @@Length < 0.0 
       js_command = "setError('Length','Invalid Length Specified');"
   else
      js_command = "setStatus('#{self.partname}');"
   end
   @@dialog.execute_script(js_command)

end

#---------------------------------------------------------------------------------------------------------
   def self.performCancel(*params)
      self.closeDialog()
   end

#---------------------------------------------------------------------------------------------------------
   def self.showDialog()
      if !defined?@@dialog
         self.InitializeDialog()
      end

      if @@UNITS == "SAE"
          @@HTMLFile = File.join(__dir__,"HTML","SAETappedHole.html")
      else
          @@HTMLFile = File.join(__dir__,"HTML","MetricTappedHole.html")
      end
      @@dialog.set_file(@@HTMLFile)
      @@dialog.show()
   end

#---------------------------------------------------------------------------------------------------------
   def self.closeDialog()
      if defined?@@dialog
         @@dialog.close()
      end
   end

#---------------------------------------------------------------------------------------------------------
#---------------------------------------------------------------------------------------------------------
#---------------------------------------------------------------------------------------------------------
   def self.partname()
      helper = ""
      if @@ThruHole == "Yes" && @@Length == @@ZERO
         #Build the helper and create the hole when helper is instantiated.
         @@Length = @@ZERO
         helper = "Helper "
      end

      "Tapped Hole #{helper}#{@@BoltSize}-#{@@TPI}-#{@@Length}-#{@@ThruHole} "
   end
#---------------------------------------------------------------------------------------------------------
   def partname()
      helper = ""
      if @thruhole == "Yes" && @length == @@ZERO
         #Build the helper and create the hole when helper is instantiated.
         @length = @@ZERO
         helper = "Helper "
      end

      "Tapped Hole #{helper}#{@boltsize}-#{@tpi}-#{@length}-#{@thruhole} "
   end

#---------------------------------------------------------------------------------------------------------
   def helper_partname()
      length = @length
      thruhole = @thruhole
      @thruhole = "Yes" 
      @length = @@ZERO

      hpn = partname

      @length = length
      @thruhole = thruhole
 
      return hpn
   end

#---------------------------------------------------------------------------------------------------------
   def initialize
        @model    = Sketchup.active_model
        @boltsize = @@BoltSize
        @tpi      = @@TPI 
        @length   = @@Length
        @thruhole = @@ThruHole
        @pitch    = @@Pitch
        @diam     = @@Diam
        @units    = @@UNITS
        @scale    = @@Scale
    end

#---------------------------------------------------------------------------------------------------------
   def create_entities( container)
     draw_me( container)
   end

#---------------------------------------------------------------------------------------------------------
#  Called by the InstancePlace observer to create a threaded hole of the correct length
#
   def create(instance, length)
      @@ZERO = "0.0".to_f
      @@BoltSize = instance.get_attribute "Fastener", "BoltSize", @@BoltSize
      @@TPI = instance.get_attribute "Fastener", "TPI", @@TPI
      @@ThruHole = instance.get_attribute "Fastener", "ThruHole", @@ThruHole
      @@Pitch = instance.get_attribute "Fastener", "Pitch", @@Pitch
      @@Diam = instance.get_attribute "Fastener", "Diam", @@Diam
      @@Scale = instance.get_attribute "Fastener", "Scale", @@Scale

      @@Length     = length

      @model    = Sketchup.active_model
      @boltsize = @@BoltSize
      @tpi      = @@TPI 
      @length   = @@Length
      @thruhole = @@ThruHole
      @pitch    = @@Pitch
      @diam     = @@Diam
      @scale    = @@Scale
      defn      = ""
	  
      #Does this Hole exist in the model already?
      #
      #partname = "Tapped Hole #{@boltsize}"
      defn = Sketchup.active_model.definitions[partname]
      if defn.nil?
         entity = create_entity(@model)
         create_entities(entity)
         defn = entity
      end
      @@Length = @@ZERO       #reset to automatic through holes...

      return defn
   end
#---------------------------------------------------------------------------------------------------------
   def create_entity(model)
      @entity = model.definitions.add(partname)
   end

#---------------------------------------------------------------------------------------------------------
   def entity
      @entity
   end

#---------------------------------------------------------------------------------------------------------
   def draw_me( container)
      puts "s=#{@scale} l=#{@length} t=#{@tpi} p=#{@pitch} d=#{@diam} b=#{@boltsize} h=#{@thruhole}"
   
      scale = @scale
      d = @diam.to_f * scale 
      p = @pitch.to_f  * scale 
      l = @length.to_f * scale

      dmin = d - (1.082532 * p)    #defined in UTS standard see: wiki UTS or Metric thread
      p8 = p / 2                   # number of 16ths
      p4 = p / 4
      p2 = p / 8
      p1 = p / 16
      dmaj2 = d / 2
      dmin2 = dmin / 2
      sin60 = 0.866025			
      cos60 = 0.500000
      arcCons = p1 / 1.732050	# Solves the radius of the arc at the bottom of each thread
      arcCenter = [dmaj2 - arcCons,0,0]
      radius = 2 * arcCons
      points = []
      points << Geom::Point3d.new(arcCenter[0] + radius,         @@ZERO, @@ZERO                            ) #    1 (
      points << Geom::Point3d.new(arcCenter[0] + sin60 * radius, @@ZERO, @@ZERO - (@@ZERO + cos60 * radius)) #    2  (
      points << Geom::Point3d.new(dmaj2,                         @@ZERO, @@ZERO - (p1                  ))    #    3   \
      points << Geom::Point3d.new(dmin2,                         @@ZERO, @@ZERO - (p4 + p2             ))    #    4    |
      points << Geom::Point3d.new(dmin2,                         @@ZERO, @@ZERO - (p8 + p2             ))    #    5    |
      points << Geom::Point3d.new(dmaj2,                         @@ZERO, @@ZERO - (p - p1              ))    #    6   /
      points << Geom::Point3d.new(arcCenter[0] + sin60 * radius, @@ZERO, @@ZERO - (p - cos60 * radius  ))    #    7  (
      points << Geom::Point3d.new(arcCenter[0] + radius,         @@ZERO, @@ZERO - (p                   ))    #    8 (
     
      if @thruhole == "Yes" && l == @@ZERO
         #Build the helper and create the hole when the helper is instantiated.
         @length = @@ZERO
         create_helper((d + radius)/scale, container)    # end result is diam + 2 * H/8 (Dont use scale in the helper!)
      else
         hole = create_tapped_hole(points, l, d, p, scale, container)
      end
   end

#---------------------------------------------------------------------------------------------------------
   def create_helper(diam, container)
      #Place the opening and then build the tapped hole once we find the other side with another component...
 
      numberofarcsegments = @@NumberOfArcSegments
      angle = (2 * Math::PI) / numberofarcsegments
      vr = Geom::Vector3d.new(0,0,1)
      tr = Geom::Transformation.rotation([0,0,0], vr, 0.0 - angle)
      p = []
      p << Geom::Point3d.new(0.0 - (diam/2.0), 0.0, 0.0)
      for i in 0..(numberofarcsegments - 2)
         p << p[i].transform(tr)
      end
      face = container.entities.add_face p
      container.entities.erase_entities face			#Keep just the edges
      container.behavior.is2d = true
      container.behavior.snapto = SnapTo_Arbitrary
      container.behavior.cuts_opening = true
      container.behavior.no_scale_mask = 127
      container.description = "EP Tapped Hole Helper for #{@boltsize}-#{@tpi}"
      container.insertion_point = Geom::Point3d.new(0.00,0.00,0.0)
      container.set_attribute "Fastener","Helper", "EP::EPFasteners::EPTappedHole"
      container.set_attribute "Fastener","Length", @@ZERO
      container.set_attribute "Fastener", "BoltSize", @boltsize
      container.set_attribute "Fastener", "Diam", @diam
      container.set_attribute "Fastener", "TPI", @tpi
      container.set_attribute "Fastener", "Pitch", @pitch
      container.set_attribute "Fastener", "ThruHole", @thruhole

      container.set_attribute "Fastener", "Scale", @scale
   end

  #---------------------------------------------------------------------------------------------------------
  def create_tapped_hole(points, length, diam, pitch, scale, container)
     time1 = Time.new
     puts "create_tapped_hole : start" + time1.inspect
	 
     numberofarcsegments = @@NumberOfArcSegments
     numberofsegments = (numberofarcsegments * (((length - (pitch/4.0)) / pitch)))
     numberofsegments = numberofsegments - 4.0 - (3.0 * numberofarcsegments)

     angle = (2 * Math::PI) / numberofarcsegments
     offset = pitch / numberofarcsegments
 
     pm = Geom::PolygonMesh.new
     pmt = Geom::PolygonMesh.new
     pmb = Geom::PolygonMesh.new
     p1 = points[0]
     p2 = points[0]
     p3 = points[0]
     p4 = points[0]
  
     vr = Geom::Vector3d.new(0,0,1)
     vp = Geom::Vector3d.new(0,0, @@ZERO - offset)
     tr = Geom::Transformation.rotation([0,0,0], vr, 0.0 - angle)
     tp = Geom::Transformation.translation(vp)
     zmin = @@ZERO

     vp4 = Geom::Vector3d.new(0,0, @@ZERO - (pitch/4.0)) #keeps from having too small of triangles on the smallest holes
     tprev = Geom::Transformation.new(vp4)
     pe1 = points[0]
     pb1 = points[0]
     pb3 = points[0]
     #Entry section of hole -------------------------------------------------------------------------------
 
     for j in 1..numberofarcsegments
        tcurr  = tprev * tp * tr
        points.each_with_index { |e,i|
           if (i+1) < points.length
              p1 = e.transform(tprev)
              p2 = points[i + 1].transform(tprev)
              p3 = e.transform(tcurr)
              p4 = points[i + 1].transform(tcurr)
  
              if (i == 0)
                 pb1 = p1.clone
                 pb3 = p3.clone
                 pb1.z = @@ZERO
                 pb3.z = @@ZERO

                 if (j == 1)
                    pe1 = pb1
                 end
                 pm.add_polygon(pb1, p1, pb3)
                 pm.add_polygon(p1, p3, pb3)
              end
   
              if j > 1
                 pm.add_polygon(p1, p2, p3)
                 pm.add_polygon(p2, p4, p3)
              else
                 # flatten the leadin
                 p1.x = pe1.x
                 p1.y = pe1.y
                 p2.x = pe1.x
                 p2.y = pe1.y
                 pm.add_polygon(p1, p2, p3)
                 pm.add_polygon(p2, p4, p3)
              end
           end
        }
        tprev = tcurr
  
     end
     container.entities.add_faces_from_mesh(pm)
     grp = container.entities.add_group

     #Regular threaded section -------------------------------------------------------------------------------------
     tstart = tprev
     blocksize = numberofarcsegments * 4
     vP = Geom::Vector3d.new(0,0, @@ZERO - (offset * blocksize))
     tP = Geom::Transformation.translation(vP)

     if ( blocksize < numberofsegments)                #block of threads...
         for j in 1..blocksize
            tcurr= tprev * tp * tr
            points.each_with_index { |e,i|
              if (i+1) < points.length
                  p1 = e.transform(tprev)
                  p2 = points[i + 1].transform(tprev)
                  p3 = e.transform(tcurr)
				  p4 = points[i + 1].transform(tcurr)
  
                  pmt.add_polygon(p1, p2, p3)
                  pmt.add_polygon(p2, p4, p3)
               end
            }
            tprev = tcurr
		 end
     end

     s = 0.0
	 nn = ((numberofsegments / blocksize) + 0.99).to_i
     while (s + blocksize) < numberofsegments            #add blocks of threads
        grp.entities.add_faces_from_mesh(pmt)
        tcurr = tprev * tP
        pmt.transform!(tP)
        s = s + blocksize
        if (s + blocksize) < numberofsegments 
           tprev = tcurr
        end

	grp.explode
        grp = container.entities.add_group
	print "."
	nn = nn - 1
     end
     puts "."
	
     #Last full thread section, less than blocksize
	 for j in 1..(numberofsegments - s).to_i
        tcurr= tprev * tp * tr
        points.each_with_index { |e,i|
          if (i+1) < points.length
              p1 = e.transform(tprev)
              p2 = points[i + 1].transform(tprev)
              p3 = e.transform(tcurr)
              p4 = points[i + 1].transform(tcurr)
  
              pmb.add_polygon(p1, p2, p3)
              pmb.add_polygon(p2, p4, p3)
           end
        }
        tprev = tcurr
     end 

     #Bottom section of hole --------------------------------------------------------------------------------------
     pc = Geom::Point3d.new(@@ZERO, @@ZERO, @@ZERO - length)
     minz = @@ZERO - length
	 
     for j in 1..numberofarcsegments
        tcurr  = tprev * tp * tr
        points.each_with_index { |e,i|
           if (i+1) < points.length
              p1 = e.transform(tprev)
              p2 = points[i + 1].transform(tprev)
              p3 = e.transform(tcurr)
              p4 = points[i + 1].transform(tcurr)
  
              p1.z = minz if p1.z < minz          # should only happen for less than 2 threads
              p2.z = minz if p2.z < minz
              p3.z = minz if p3.z < minz
              p4.z = minz if p4.z < minz
  
              if j < numberofarcsegments
                 pmb.add_polygon(p1, p2, p3)
                 pmb.add_polygon(p2, p4, p3)
              else
                 p3.x = pe1.x
                 p3.y = pe1.y
                 p4.x = pe1.x
                 p4.y = pe1.y
                 pmb.add_polygon(p1, p2, p3)
                 pmb.add_polygon(p2, p4, p3)
              end
  
              if (i+2) == points.length
                 p1 = p2.clone
                 p3 = p4.clone
                 p1.z = minz
                 p3.z = minz
  
                 if (j == 1)
                    pe1 = p1
                 end
                 pmb.add_polygon(p2, p1, p3)
                 pmb.add_polygon(p4, p2, p3)
              end
           end
        }
        tprev = tcurr
  
     end
  
     # Generate the faces ---------------------------------------------------------------------------------------------
     #grp = container.entities.add_group
     grp.entities.add_faces_from_mesh(pmb)
     edges = grp.entities.select { |e| e.typename == "Edge" &&  e.start.position.z == (@@ZERO - length) && e.end.position.z == (@@ZERO - length) }

     time1 = Time.new
     puts "create_tapped_hole Geometry complete: " + time1.inspect

     if @thruhole == "No"
        edges[0].find_faces if edges.count > 0
     else
        helpername = helper_partname
        container.set_attribute "Fastener","Helper",helpername
        container.set_attribute "Fastener","Length",length
     end
     grp.explode

     newscale = 1.0000000 / scale
     if newscale != 1.000000 then
        t = Geom::Transformation.scaling newscale
	container.entities.transform_entities t, container.entities.select { true }
     end

     edgestosoften =  container.entities.select { |e| e.typename == "Edge" and (!e.soft? or !e.hidden?)}	 
     edgestosoften.each { |e| e.soft=e.hidden=true }
	 
     container.behavior.is2d = true
     container.behavior.snapto = SnapTo_Arbitrary
     container.behavior.cuts_opening = true

     container.behavior.no_scale_mask = 127
     container.description = "EP Threaded Hole #{@boltsize} #{@tpi} Len=#{@length}"
     container.insertion_point = Geom::Point3d.new(@@ZERO,@@ZERO,@@ZERO) 
     time1 = Time.new
     puts "create_tapped_hole done: " + time1.inspect 

  end

end  #class
#=============================================================================

end #module EPFasteners
end #module EP